@@ -1,9 +1,13 @@
-"""Copyright (c) 2006-2018 by the respective copyright holders.
+"""Copyright (c) 2006-2020 by the respective copyright holders.
 
-All rights reserved. This program and the accompanying materials
-are made available under the terms of the Eclipse Public License v1.0
-which accompanies this distribution, and is available at
-http://www.eclipse.org/legal/epl-v10.html
+See the NOTICE file(s) distributed with this work for additional
+information.
+
+This program and the accompanying materials are made available under the
+terms of the Eclipse Public License 2.0 which is available at
+http://www.eclipse.org/legal/epl-2.0
+
+SPDX-License-Identifier: EPL-2.0
 
 Contributors:
   Andre Lengwenus - port to Python and further improvements
@@ -12,6 +16,7 @@ Contributors:
 
 import asyncio
 import logging
+from typing import Any, Awaitable, Callable, Dict, Optional, Tuple, Union
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -20,10 +25,12 @@ _LOGGER = logging.getLogger(__name__)
 DEFAULT_TIMEOUT_MSEC = 3500
 
 
+TimeoutCallback = Union[Callable[..., None], Callable[..., Awaitable[None]]]
+
+
 class TimeoutRetryHandler:
     """Manage timeout and retry logic for an LCN request.
 
-    :param           loop:           The asyncio event loop
     :param    int    num_tries:      The maximum number of tries until the
                                      request is marked as failed (-1 means
                                      forever)
@@ -31,24 +38,25 @@ class TimeoutRetryHandler:
 
     """
 
-    def __init__(self, loop, num_tries=3, timeout_msec=DEFAULT_TIMEOUT_MSEC):
+    def __init__(self, num_tries: int = 3, timeout_msec: int = DEFAULT_TIMEOUT_MSEC):
         """Construct TimeoutRetryHandler."""
-        self.loop = loop
         self.num_tries = num_tries
         self.timeout_msec = timeout_msec
-        self._timeout_callback = None
-        self._timeout_args = []
-        self._timeout_kwargs = {}
-        self.timeout_loop_task = None
+        self._timeout_callback: Optional[TimeoutCallback] = None
+        self._timeout_args: Tuple[Any, ...] = ()
+        self._timeout_kwargs: Dict[str, Any] = {}
+        self.timeout_loop_task: Optional[asyncio.Task[None]] = None
 
-    def set_timeout_msec(self, timeout_msec):
+    def set_timeout_msec(self, timeout_msec: int) -> None:
         """Set the timeout in milliseconds.
 
         :param    int    timeout_msec:    The timeout in milliseconds
         """
         self.timeout_msec = timeout_msec
 
-    def set_timeout_callback(self, timeout_callback, *timeout_args, **timeout_kwargs):
+    def set_timeout_callback(
+        self, timeout_callback: Any, *timeout_args: Any, **timeout_kwargs: Any
+    ) -> None:
         """Timeout_callback function is called, if timeout expires.
 
         Function has to take one argument:
@@ -58,21 +66,22 @@ class TimeoutRetryHandler:
         self._timeout_args = timeout_args
         self._timeout_kwargs = timeout_kwargs
 
-    def activate(self):
+    def activate(self) -> None:
         """Schedule the next activation."""
-        self.loop.create_task(self.async_activate())
+        asyncio.create_task(self.async_activate())
 
-    async def async_activate(self):
+    async def async_activate(self) -> None:
         """Clean start of next timeout_loop."""
         if self.is_active():
             await self.cancel()
-        self.timeout_loop_task = self.loop.create_task(self.timeout_loop())
+        self.timeout_loop_task = asyncio.create_task(self.timeout_loop())
 
-    async def done(self):
+    async def done(self) -> None:
         """Signal the completion of the TimeoutRetryHandler."""
-        await self.timeout_loop_task
+        if self.timeout_loop_task is not None:
+            await self.timeout_loop_task
 
-    async def cancel(self):
+    async def cancel(self) -> None:
         """Must be called when a response (requested or not) is received."""
         if self.timeout_loop_task is not None:
             self.timeout_loop_task.cancel()
@@ -81,17 +90,19 @@ class TimeoutRetryHandler:
             except asyncio.CancelledError:
                 pass
 
-    def is_active(self):
+    def is_active(self) -> bool:
         """Check whether the request logic is active."""
         if self.timeout_loop_task is None:
             return False
         return not self.timeout_loop_task.done()
 
-    async def on_timeout(self, failed=False):
+    async def on_timeout(self, failed: bool = False) -> None:
         """Is called on timeout of TimeoutRetryHandler."""
         if self._timeout_callback is not None:
             if asyncio.iscoroutinefunction(self._timeout_callback):
-                await self._timeout_callback(
+                # mypy fails to notice that `asyncio.iscoroutinefunction`
+                # separates await-callable from ordinary callables.
+                await self._timeout_callback(  # type: ignore
                     failed, *self._timeout_args, **self._timeout_kwargs
                 )
             else:
@@ -99,8 +110,10 @@ class TimeoutRetryHandler:
                     failed, *self._timeout_args, **self._timeout_kwargs
                 )
 
-    async def timeout_loop(self):
+    async def timeout_loop(self) -> None:
         """Timeout / retry loop."""
+        if self.timeout_loop_task is None:
+            return
         tries_left = self.num_tries
         while (tries_left > 0) or (tries_left == -1):
             if not self.timeout_loop_task.done():
